/* Here's the implementation.                                               */
/*                                                                          */
/* <*>=                                                                     */
#include <assert.h>
/* % -*- mode: Noweb; noweb-code-mode: c-mode -*-                           */
/*                                                                          */
/* \section{Temporary null termination of strings}                          */
/* This hack allows you to put a zero after a string \emph{temporarily},    */
/* i.e., just long enough to use [[printf]].                                */
/* Calling [[pushchar(p)]] is equivalent to writing \mbox{[[*p = '\0']]},   */
/* except that calling [[popchar()]] undoes the effect of the last          */
/* [[pushchar]].                                                            */
/* The stack is shallow, and the program halts with an assertion failure    */
/* if it overflows.                                                         */
/*                                                                          */
/* <header>=                                                                */
extern void pushchar(char *p);
extern void popchar(void);
static char stack[3];
static char *pstack[3];
static int next = 0;
void pushchar(char *p) {
  assert(next < sizeof(stack));
  stack[next] = *p;
  pstack[next++] = p;
  *p = 0;
}
void popchar(void) {
  char *p;
  assert(next > 0);
  p = pstack[--next];
  *p = stack[next];
}
